@@ -1,6 +1,6 @@
 #!/usr/bin/python
 
-# Copyright 2003-2010 Gentoo Foundation
+# Copyright 2003-2016 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
 
 
@@ -120,7 +120,7 @@ def printUsage(_error=None, help=None):
 		print( yellow(" -i, --interactive")+
 			"         - ask confirmation before deletions", file=out)
 		print( yellow(" -n, --package-names")+
-			"       - protect all versions (when --deep", file=out)
+			"       - protect all versions (when --deep)", file=out)
 		print( yellow(" -p, --pretend")+
 			"             - only display what would be cleaned", file=out)
 		print( yellow(" -q, --quiet")+
@@ -147,7 +147,8 @@ def printUsage(_error=None, help=None):
 	or help in ('all','packages'):
 		print( "Available", yellow("options"),"for the",
 				green("packages"),"action:", file=out)
-		print( yellow(" NONE  :)"), file=out)
+		print( yellow(" -i, --ignore-failure")+
+			"             - ignore failure to locate PKGDIR", file=out)
 		print( file=out)
 	if _error in ('distfiles-options', 'merged-distfiles-options') \
 	or help in ('all','distfiles'):
@@ -262,6 +263,8 @@ def parseArgs(options={}):
 				options['size-limit'] = parseSize(a)
 			elif o in ("-v", "--verbose") and not options['quiet']:
 					options['verbose'] = True
+			elif o in ("-i", "--ignore-failure"):
+				options['ignore-failure'] = True
 			else:
 				return_code = False
 		# sanity check of --deep only options:
@@ -286,8 +289,8 @@ def parseArgs(options={}):
 		"time-limit=", "package-names", "help", "version",  "verbose"]
 	getopt_options['short']['distfiles'] = "fs:"
 	getopt_options['long']['distfiles'] = ["fetch-restricted", "size-limit="]
-	getopt_options['short']['packages'] = ""
-	getopt_options['long']['packages'] = [""]
+	getopt_options['short']['packages'] = "i"
+	getopt_options['long']['packages'] = ["ignore-failure"]
 	# set default options, except 'nocolor', which is set in main()
 	options['interactive'] = False
 	options['pretend'] = False
@@ -300,6 +303,7 @@ def parseArgs(options={}):
 	options['fetch-restricted'] = False
 	options['size-limit'] = 0
 	options['verbose'] = False
+	options['ignore-failure'] = False
 	# if called by a well-named symlink, set the acction accordingly:
 	action = None
 	# temp print line to ensure it is the svn/branch code running, etc..
@@ -444,7 +448,7 @@ def main():
 	"""Parse command line and execute all actions."""
 	# set default options
 	options = {}
-	options['nocolor'] = (port_settings["NOCOLOR"] in ('yes','true')
+	options['nocolor'] = (port_settings.get("NOCOLOR") in ('yes','true')
 		or not sys.stdout.isatty())
 	if options['nocolor']:
 		pp.output.nocolor()
