@@ -532,50 +532,57 @@ def findPackages(
 		test = os.listdir(pkgdir)
 		del test
 	except EnvironmentError as er:
+		if options['ignore-failure']:
+			exit(0)
 		print( pp.error("Error accessing PKGDIR." ), file=sys.stderr)
 		print( pp.error("(Check your make.conf file and environment)."), file=sys.stderr)
 		print( pp.error("Error: %s" %str(er)), file=sys.stderr)
 		exit(1)
-	for root, dirs, files in os.walk(pkgdir):
-		if root[-3:] == 'All':
+
+	# if portage supports FEATURES=binpkg-multi-instance, then
+	# cpv_all can return multiple instances per cpv, where
+	# instances are distinguishable by some extra attributes
+	# provided by portage's _pkg_str class
+	bin_dbapi = portage.binarytree(pkgdir=pkgdir, settings=var_dbapi.settings).dbapi
+	for cpv in bin_dbapi.cpv_all():
+		mtime = int(bin_dbapi.aux_get(cpv, ['_mtime_'])[0])
+		if time_limit and mtime >= time_limit:
+			# time-limit exclusion
 			continue
-		for file in files:
-			if not file[-5:] == ".tbz2":
-				# ignore non-tbz2 files
-				continue
-			path = os.path.join(root, file)
-			category = os.path.split(root)[-1]
-			cpv = category+"/"+file[:-5]
-			st = os.lstat(path)
-			if time_limit and (st[stat.ST_MTIME] >= time_limit):
-				# time-limit exclusion
-				continue
-			# dict is cpv->[files] (2 files in general, because of symlink)
-			clean_me[cpv] = [path]
-			#if os.path.islink(path):
-			if stat.S_ISLNK(st[stat.ST_MODE]):
-				clean_me[cpv].append(os.path.realpath(path))
+		# dict is cpv->[pkgs] (supports binpkg-multi-instance)
+		clean_me.setdefault(cpv, []).append(cpv)
+
 	# keep only obsolete ones
-	if destructive:
-		dbapi = var_dbapi
-		if package_names:
-			cp_all = dict.fromkeys(dbapi.cp_all())
-		else:
-			cp_all = {}
+	if destructive and package_names:
+		cp_all = dict.fromkeys(var_dbapi.cp_all())
 	else:
-		dbapi = port_dbapi
 		cp_all = {}
 	for cpv in list(clean_me):
 		if exclDictMatchCP(exclude,portage.cpv_getkey(cpv)):
 			# exclusion because of the exclude file
 			del clean_me[cpv]
 			continue
-		if dbapi.cpv_exists(cpv):
-			# exclusion because pkg still exists (in porttree or vartree)
+		if not destructive and port_dbapi.cpv_exists(cpv):
+			# exclusion because pkg still exists (in porttree)
 			del clean_me[cpv]
 			continue
-		if portage.cpv_getkey(cpv) in cp_all:
+		if destructive and var_dbapi.cpv_exists(cpv):
+			buildtime = var_dbapi.aux_get(cpv, ['BUILD_TIME'])[0]
+			clean_me[cpv] = [pkg for pkg in clean_me[cpv]
+				# only keep path if BUILD_TIME is identical with vartree
+				if bin_dbapi.aux_get(pkg, ['BUILD_TIME'])[0] != buildtime]
+			if not clean_me[cpv]:
+				# nothing we can clean for this package
+				del clean_me[cpv]
+				continue
+		if portage.cpv_getkey(cpv) in cp_all and port_dbapi.cpv_exists(cpv):
 			# exlusion because of --package-names
 			del clean_me[cpv]
 
+	# the getname method correctly supports FEATURES=binpkg-multi-instance,
+	# allowing for multiple paths per cpv (the API used here is also compatible
+	# with older portage which does not support binpkg-multi-instance)
+	for cpv, pkgs in clean_me.items():
+		clean_me[cpv] = [bin_dbapi.bintree.getname(pkg) for pkg in pkgs]
+
 	return clean_me
