@@ -17,6 +17,8 @@ __author__ = "Marius Mauch <genone@gentoo.org>"
 
 
 import sys
+if sys.hexversion < 0x3000000:
+	from io import open
 import os
 try:
     from urllib import urlopen
@@ -35,9 +37,11 @@ if sys.version_info[0:2] < (2,3):
 
 try:
 	import portage
+	from portage import _encodings, _unicode_decode, _unicode_encode
 except ImportError:
 	sys.path.insert(0, "/usr/lib/portage/pym")
 	import portage
+	from portage import _encodings, _unicode_decode, _unicode_encode
 
 
 # Note: the space for rgt and rlt is important !!
@@ -557,8 +561,8 @@ class Glsa:
 		# <revised count="2">2007-12-30</revised>
 		revisedEl = myroot.getElementsByTagName("revised")[0]
 		self.revised = getText(revisedEl, format="strip")
-		count = revisedEl.attributes.get("count")
-		if count is None:
+		count = revisedEl.getAttribute("count")
+		if not count:
 			if self.revised.find(":") >= 0:
 				(self.revised, count) = self.revised.split(":")
 			else:
@@ -702,7 +706,9 @@ class Glsa:
 		@returns:	None
 		"""
 		if not self.isInjected():
-			checkfile = open(self.config["CHECKFILE"], "a+")
+			checkfile = open(_unicode_encode(self.config["CHECKFILE"],
+				encoding=_encodings['fs']), mode="a+",
+				encoding=_encodings['content'])
 			checkfile.write(self.nr+"\n")
 			checkfile.close()
 		return None
